package com.xy.job;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import com.xy.collections.map.JHashMap;
import com.xy.collections.map.JMap;
import com.xy.entity.SysDictRedis;
import com.xy.mapper.DeviceChargingMapper;
import com.xy.sys.EnumMercCostMsgConfig;
import com.xy.utils.*;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;

/**
 * 设备管理费job
 */
@Slf4j
@Component
@AllArgsConstructor
public class DeviceChargingJob {

    private DeviceChargingMapper deviceChargingMapper;

    /**
     * 设备管理费过期消息
     *
     * @return
     */
    @XxlJob("deviceChargingTimeout")
    public ReturnT<String> deviceChargingTimeout() {
        String now = DataTime.getSring();
        Map<String, SysDictRedis> stringSysDictRedisMap = SysDictUtils.get(EnumMercCostMsgConfig.Code.CODE.getCode());
        String tel = stringSysDictRedisMap.get(EnumMercCostMsgConfig.CUSTOM_TEL.getCode()).toString();
        //即将过期
        FunctionUtils.NoParamsNoResult timeout1 = () -> {
            int day = Integer.valueOf(stringSysDictRedisMap.get(EnumMercCostMsgConfig.DEVICE_COST_DAY.getCode()).toString());
            String end = DataTime.getStringAround(0, 0, day, 0, 0, 0);
            deviceChargingTimeout1(now, end, 1, 1000, tel);
        };
        //欠费
        FunctionUtils.NoParamsNoResult timeout2 = () -> deviceChargingTimeout2(now, now, 1, 1000, tel);
        timeout1.run();
        timeout2.run();
        return ReturnT.SUCCESS;
    }

    /**
     * 即将过期
     *
     * @param begin
     * @param end
     * @param current
     * @param size
     * @param tel
     */
    private void deviceChargingTimeout1(String begin, String end, long current, long size, String tel) {
        //查询数据
        IPage<Long> longIPage = deviceChargingMapper.deviceChargingTimeout(Page.of(current, size), begin, end);
        List<Long> mercIds = longIPage.getRecords();
        if (!Emptys.check(mercIds)) {
            return;
        }
        //发送消息
        JMap<String, Object> params = new JHashMap<String, Object>()
                .set("value", "您有设备管理费即将到期")
                .set("phone", tel);
        MsgUtils.sendMsg(15L, mercIds)
                .official(userInfo -> new MsgUtils.MsgInfo(userInfo.getMpOpenid(), params))
                .mini(userInfo -> new MsgUtils.MsgInfo(userInfo.getWechatOpenid(), params))
                .builder();
        //下一页
        current++;
        deviceChargingTimeout1(begin, end, current, size, tel);
    }

    /**
     * 已欠费
     *
     * @param end
     * @param thisTime
     * @param current
     * @param size
     * @param tel
     */
    private void deviceChargingTimeout2(String end, String thisTime, long current, long size, String tel) {
        //查询数据
        IPage<Long> longIPage = deviceChargingMapper.deviceChargingTimeout(Page.of(current, size), null, end);
        List<Long> mercIds = longIPage.getRecords();
        if (!Emptys.check(mercIds)) {
            return;
        }
        //发送消息
        JMap<String, Object> params = new JHashMap<String, Object>()
                .set("time0", thisTime)
                .set("type", "设备管理费")
                .set("time1", "")
                .set("phone", tel);
        MsgUtils.sendMsg(15L, mercIds)
                .official(userInfo -> new MsgUtils.MsgInfo(userInfo.getMpOpenid(), params))
                .mini(userInfo -> new MsgUtils.MsgInfo(userInfo.getWechatOpenid(), params))
                .builder();
        //下一页
        current++;
        deviceChargingTimeout2(end, thisTime, current, size, tel);
    }

}
